@@ -41,7 +41,8 @@ typedef enum
 	P_INTEGRAL,
 	P_SIMPAD,
 	P_SIMPAD_NEW,
-    P_GENERIC
+    P_GENERIC,
+	P_SYSFS
 }t_platform;
 
 
@@ -79,16 +80,30 @@ char IpaqModel = -1;
 #define FL_IOCTL_STEP_CONTRAST    100
 
 /* Simpad, a little bit nasty - PWM reg. is set directly */
-
 #define SIMPAD_BACKLIGHT_REG		"/proc/driver/mq200/registers/PWM_CONTROL"
 #define SIMPAD_BACKLIGHT_MASK		0x00a10044
 
 /* Simpad, new interface */
 #define SIMPAD_BACKLIGHT_REG_NEW	"/proc/driver/mq200/backlight"
 
-/* Generic backight */
+/* Generic backlight */
 #define GENERIC_PROC_DRIVER "/proc/driver/backlight"
 
+/* Linux 2.6 sysfs interface */
+#define SYS_STATE_ON  0
+#define SYS_STATE_OFF 4
+static char *SYS_BRIGHTNESS = NULL;
+static char *SYS_POWER = NULL;
+static char *SYS_LCDPOWER = NULL;
+
+const char *sysbdevs[] = 
+{
+	"/sys/class/backlight/sa1100fb",
+	"/sys/class/backlight/pxafb",
+	NULL
+};
+
+
 GtkWidget *slider_window;
 GtkWidget *window, *slider;
 GtkWidget *icon;
@@ -108,6 +123,8 @@ t_platform platform = P_NONE;
 static t_platform
 detect_platform(void)
 {
+	int i = 0;
+	
 	if (!access(TS_DEV,R_OK))
 		return P_IPAQ;
 	if (!access(PROC_LIGHT,R_OK))
@@ -122,10 +139,67 @@ detect_platform(void)
 		return P_SIMPAD;
 	if (!access(GENERIC_PROC_DRIVER,R_OK))
 		return P_GENERIC;
+	while (sysbdevs[i])
+	{
+		if (!access(sysbdevs[i], R_OK))
+		{
+			SYS_BRIGHTNESS = g_strdup_printf("%s/brightness", sysbdevs[i]);
+			SYS_POWER = g_strdup_printf("%s/power", sysbdevs[i]);
+			SYS_LCDPOWER = g_strdup_printf("/sys/class/lcd/%s/power", strrchr(sysbdevs[i],'/')+1);
+			return P_SYSFS;
+		}
+		i++;
+	}
 	return P_NONE;
 }
 
 
+int 
+sysfs_set_level(int level)
+{
+  FILE *f_light;
+  
+  f_light = fopen(SYS_BRIGHTNESS, "w");
+  if (f_light != NULL)
+  {
+    fprintf(f_light,"%d\n", level);
+  	fclose(f_light);
+  }
+  else
+	  return -1;
+  
+  f_light = fopen(SYS_POWER, "w");
+  if (f_light != NULL)
+  {
+    fprintf(f_light,"%d\n",  level ? SYS_STATE_ON : SYS_STATE_OFF);
+  	fclose(f_light);
+  }
+  f_light = fopen(SYS_LCDPOWER, "w");
+  if (f_light != NULL)
+  {
+    fprintf(f_light,"%d\n",  level ? SYS_STATE_ON : SYS_STATE_OFF);
+  	fclose(f_light);
+  }
+  
+  return level;
+}
+
+int 
+sysfs_get_level(void)
+{
+  FILE *f_light;
+  int level;
+  
+  f_light = fopen(SYS_BRIGHTNESS, "r");
+  if (f_light != NULL)
+  {
+  	fscanf(f_light,"%d", &level);
+  	fclose(f_light);
+	return level;
+  }
+  return -1;
+}  
+
 int 
 generic_set_level(int level)
 {
@@ -163,6 +237,7 @@ simpad_new_set_level(int level)
 {
   FILE *f_light;
   
+  if (level < 1) level = 1;
   f_light = fopen(SIMPAD_BACKLIGHT_REG_NEW,"w");
   if (f_light != NULL)
   {
@@ -197,6 +272,7 @@ simpad_set_level(int level)
   int val;
   FILE *f_light;
   
+  if (level < 1) level = 1;
   f_light = fopen(SIMPAD_BACKLIGHT_REG,"w");
   if (f_light != NULL)
   {
@@ -426,6 +502,12 @@ set_level (int level)
 	case P_SIMPAD:
 		return simpad_set_level(level);
 	break;
+	case P_GENERIC:
+		return generic_set_level(level);
+	break;
+	case P_SYSFS:
+		return sysfs_set_level(level);
+	break;
 	default:
 		return 0;
 	break;
@@ -468,6 +550,9 @@ read_old_level (void)
 	case P_GENERIC:
 		return generic_get_level();
 	break;
+	case P_SYSFS:
+		return sysfs_get_level();
+	break;
 	default:
 		return 0;
 	break;
@@ -606,7 +691,7 @@ main (int argc, char **argv)
       break;
   }
   else if (platform == P_GENERIC)
-         slider = gtk_vscale_new_with_range (0, 32, 1);
+         slider = gtk_vscale_new_with_range (1, 32, 1);
        else  
   	     slider = gtk_vscale_new_with_range (0, 255, 1);
 
